@@ -10,6 +10,8 @@
 
 #include <botan/eme.h>
 
+BOTAN_FUTURE_INTERNAL_HEADER(eme_pkcs.h)
+
 namespace Botan {
 
 /**
@@ -19,7 +21,7 @@ class BOTAN_PUBLIC_API(2,0) EME_PKCS1v15 final : public EME
    {
    public:
       size_t maximum_input_size(size_t) const override;
-   private:
+
       secure_vector<uint8_t> pad(const uint8_t[], size_t, size_t,
                              RandomNumberGenerator&) const override;
 
